﻿using Avalonia.Controls.Shapes;
using Avalonia.Controls;
using Avalonia.Media;
using Avalonia.Threading;
using avalonia_second_practic.Models;
using System;
using ReactiveUI;

namespace avalonia_second_practic.ViewModels
{
    public class PageOneViewModel : ReactiveObject
    {
        private DispatcherTimer _timer = new DispatcherTimer();

        private string _userEmal;
        private string _userPassword;
        private string _resultText;
        private bool _authVisible = true;
        private Canvas _canvas;
        private string _captchaText = "";
        private bool _captchaVisible = false;
        private string _userCaptchaText;
        private bool _enableTB = true;
        private int _countSeconds = 10;
        private string _captchaErrorText;
        private bool _signalToNext = false;

        public string UserEmal { get => _userEmal; set => this.RaiseAndSetIfChanged(ref _userEmal, value); }
        public string UserPassword { get => _userPassword; set => this.RaiseAndSetIfChanged(ref _userPassword, value); }
        public string ResultText { get => _resultText; set => this.RaiseAndSetIfChanged(ref _resultText, value); }
        public bool AuthVisible { get => _authVisible; set => this.RaiseAndSetIfChanged(ref _authVisible, value); }
        public Canvas CanvasGen { get => _canvas; set => this.RaiseAndSetIfChanged(ref _canvas, value); }
        public bool CaptchaVisible { get => _captchaVisible; set => this.RaiseAndSetIfChanged(ref _captchaVisible, value); }
        public string UserCaptchaText { get => _userCaptchaText; set => this.RaiseAndSetIfChanged(ref _userCaptchaText, value); }
        public bool EnableTB { get => _enableTB; set => this.RaiseAndSetIfChanged(ref _enableTB, value); }
        public string CaptchaErrorText { get => _resultText; set => this.RaiseAndSetIfChanged(ref _resultText, value); }
        public bool SignalToNext { get => _signalToNext; set => this.RaiseAndSetIfChanged(ref _signalToNext, value); }

        public void AuthUser()
        {
            Random rnd = new Random();
            UserData ud = new UserData();
            Authorization au = new Authorization();

            if (string.IsNullOrEmpty(UserEmal))
            {
                PrintResult("Поле email пустое!");
            }

            if (string.IsNullOrEmpty(UserPassword))
            {
                PrintResult("Поле пароля пустое!");
            }

            if (string.IsNullOrEmpty(UserEmal) && string.IsNullOrEmpty(UserPassword))
            {
                PrintResult("Поля для данных пустые!");
            }

            if (!string.IsNullOrEmpty(UserEmal) && !string.IsNullOrEmpty(UserPassword) && !au.VeryfiUserWithEmail(UserEmal, UserPassword, ud.Login, ud.Password))
            {
                PrintResult("Не правильные логин или пароль!");
                AuthVisible = false;

                if (CaptchaVisible)
                {
                    DelayCaptcha();
                }

                if (!_timer.IsEnabled)
                {
                    _captchaText = "";
                    CreateCaptcha();
                    CaptchaVisible = true;
                }
            }
            if (!string.IsNullOrEmpty(UserEmal) && !string.IsNullOrEmpty(UserPassword) && au.VeryfiUserWithEmail(UserEmal, UserPassword, ud.Login, ud.Password))
            {
                if (CaptchaVisible)
                {
                    _captchaText = "";
                    CaptchaVisible = false;
                }
                SignalToNext = true;
            }
        }

        public void PrintResult(string message)
        {
            ResultText = message;
        }

        public void CreateCaptcha()
        {
            Random rnd = new Random();

            SolidColorBrush color = new SolidColorBrush(Color.FromRgb(Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(256))));

            int width = 400;
            int height = 150;

            Canvas canvas = new Canvas()
            {
                Width = width,
                Height = height,
                Background = color
            };

            //вернуть рандом для генерации каптчи - rnd.Next(7, 10)
            //для простоты проверки установлено значение - 1

            for (int i = 0; i < rnd.Next(7, 10); i++)
            {
                if (rnd.Next() > (Int32.MaxValue / 2))
                {
                    if (rnd.Next() > (Int32.MaxValue / 2))
                    {
                        _captchaText += ((char)(rnd.Next('A', 'A' + 26))).ToString();
                    }
                    else
                    {
                        _captchaText += ((char)(rnd.Next('a', 'a' + 26))).ToString();
                    }
                }
                else
                {
                    _captchaText += ((char)(rnd.Next('0', '9' + 1))).ToString();
                }
            }

            //генерация букв каптчи
            int paddingCounter = 0;
            foreach (var item in _captchaText.ToCharArray())
            {
                paddingCounter += 20;
                TextBlock tb = new TextBlock()
                {
                    Text = item.ToString(),
                    FontSize = rnd.Next(40, 60),
                    Foreground = new SolidColorBrush(Color.FromRgb(Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(256)))),
                    Padding = new Avalonia.Thickness(rnd.Next(paddingCounter, paddingCounter += 10), rnd.Next(10, 50)),
                    FontWeight = (FontWeight)(rnd.Next(2, 17) * 50),
                    FontStyle = (FontStyle)rnd.Next(0, 2)
                };

                canvas.Children.Add(tb);
            }

            //генерация линий каптчи
            for (int i = 0; i < rnd.Next(20, 30); i++)
            {
                Line line = new Line()
                {
                    StartPoint = new Avalonia.Point(rnd.Next(0, width), rnd.Next(0, height)),
                    EndPoint = new Avalonia.Point(rnd.Next(0, width), rnd.Next(0, height)),
                    StrokeThickness = rnd.Next(1, 5),
                    Stroke = new SolidColorBrush(Color.FromRgb(Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(256)), Convert.ToByte(rnd.Next(256)))),
                };
                canvas.Children.Add(line);
            }

            CanvasGen = canvas;
        }

        private void PrintTimer(object sender, EventArgs e)
        {
            PrintResult("До разблокировки осталось: " + (_countSeconds--).ToString());
            if (_countSeconds < 0)
            {
                _timer.Stop();
                _countSeconds = 10;
                EnableTB = true;
                PrintResult("");
                CreateCaptcha();
                CaptchaVisible = true;
            }
        }

        public void PrintErrorCaptch(string message)
        {
            CaptchaErrorText = message;
        }

        private void DelayCaptcha()
        {
            CanvasGen.Children.Clear();
            _captchaText = "";
            CaptchaVisible = false;
            UserPassword = "";
            UserEmal = "";
            UserCaptchaText = "";
            EnableTB = false;
            _timer.Interval = TimeSpan.FromSeconds(1);
            _timer.Tick += new EventHandler(PrintTimer);
            _timer.Start();
        }

        public void CheckCaptcha()
        {
            if (string.IsNullOrEmpty(UserCaptchaText))
            {
                PrintErrorCaptch("Пустая каптча!");
            }

            if (!string.IsNullOrEmpty(UserCaptchaText) && UserCaptchaText.Trim() == _captchaText)
            {
                AuthUser();
            }

            if (!string.IsNullOrEmpty(UserCaptchaText) && UserCaptchaText.Trim() != _captchaText)
            {
                PrintErrorCaptch("Не верная каптча!");
                DelayCaptcha();
            }
        }
    }
}
