﻿using avalonia_second_practic.Models;
using ReactiveUI;
using System.Collections.Generic;

namespace avalonia_second_practic.ViewModels
{
    public class PageTwoViewModel : ReactiveObject
    {
        private string _startOperation = CalculateOperation.operationsFullname[0];
        private string _startShortOperator = "+";
        private string _numberOne;
        private string _numberTwo;
        public string _errorText = "";
        private Calculator _calculator = new Calculator();
        private bool _calculateEnable = false;
        private string _result = "";

        public List<string> FullNameOperations => CalculateOperation.operationsFullname;
        public string StartOperation
        {
            get => _startOperation;
            set
            {
                if (value != _startOperation)
                {
                    ShortNameOperation = CalculateOperation.operationsShortname[CalculateOperation.operationsFullname.IndexOf(value)];
                }

                this.RaiseAndSetIfChanged(ref _startOperation, value);
            }
        }
        public string ShortNameOperation { get => _startShortOperator; set => this.RaiseAndSetIfChanged(ref _startShortOperator, value); }
        public string ErrorText { get => _errorText; set => this.RaiseAndSetIfChanged(ref _errorText, value); }
        public string NumberOne
        {
            get => _numberOne;
            set
            {
                this.RaiseAndSetIfChanged(ref _numberOne, value);

                if (string.IsNullOrEmpty(_numberOne))
                {
                    ErrorText = "Поля для первого значения пустое!";
                    CalculateEnable = false; Result = "";
                }
                else
                {
                    if (CheckToConvertDouble(_numberOne))
                    {
                        _calculator.NumberOne = double.Parse(_numberOne);
                        ErrorText = "";
                        if (_calculator.NumberTwo != null)
                        {
                            CalculateEnable = true;
                        }
                    }
                    else
                    {
                        ErrorText = "Первое значение нельзя преобразовать к числу!";
                        CalculateEnable = false; Result = "";
                    }
                }
            }
        }
        public string NumberTwo
        {
            get => _numberTwo;
            set
            {
                this.RaiseAndSetIfChanged(ref _numberTwo, value);

                if (string.IsNullOrEmpty(_numberTwo) && _calculator.NumberTwo != null)
                {
                    ErrorText = "Поля для второго значения пустое!";
                    CalculateEnable = false; Result = "";
                }
                else
                {
                    if (CheckToConvertDouble(_numberTwo))
                    {
                        if (ShortNameOperation == CalculateOperation.operationsShortname[2])
                        {
                            if (double.Parse(_numberTwo) == 0)
                            {
                                ErrorText = "Делить на 0 нельзя!";
                                CalculateEnable = false; Result = "";
                            }
                            else
                            {
                                _calculator.NumberTwo = double.Parse(_numberTwo);
                                ErrorText = "";
                                if (_calculator.NumberOne != null)
                                {
                                    CalculateEnable = true;
                                }
                            }
                        }
                        else
                        {
                            _calculator.NumberTwo = double.Parse(_numberTwo);
                            ErrorText = "";
                            if (_calculator.NumberOne != null)
                            {
                                CalculateEnable = true;
                            }
                        }
                    }
                    else
                    {
                        ErrorText = "Второе значение нельзя преобразовать к числу!";
                        CalculateEnable = false; Result = "";
                    }
                }
            }
        }
        public bool CalculateEnable
        {
            get => _calculateEnable;
            set
            {
                this.RaiseAndSetIfChanged(ref _calculateEnable, value);
            }
        }
        public string Result { get => _result; set => this.RaiseAndSetIfChanged(ref _result, value); }

        /// <summary>
        /// Проверка, возможно ли конвертировать в double
        /// </summary>
        /// <param name="number"></param>
        /// <returns></returns>
        private bool CheckToConvertDouble(string number)
        {
            try
            {
                double.Parse(number);
                return true;
            }
            catch
            {
                return false;
            }
        }

        public void Calculate()
        {
            if (_calculator.NumberOne == null || _calculator.NumberTwo == null)
            {
                ErrorText = "Какое-то из полей пустое!";
                CalculateEnable = false; Result = "";
            }
            else if (!CheckToConvertDouble(_numberTwo) && !CheckToConvertDouble(_numberOne))
            {
                ErrorText = "Какое-то из значений нельзя преобразовать к числу!";
                CalculateEnable = false; Result = "";
            }
            else if (ShortNameOperation == CalculateOperation.operationsShortname[2])
            {
                if (double.Parse(_numberTwo) == 0)
                {
                    ErrorText = "Делить на 0 нельзя!";
                    CalculateEnable = false; Result = "";
                }
                CalculateStart(StartOperation);
            }
            else
            {
                CalculateStart(StartOperation);
            }
        }

        private void CalculateStart(string operation)
        {
            switch (operation)
            {
                case "Сложение": Result = "Ответ:\n" + _calculator.Sum().ToString(); break;
                case "Вычитание": Result = "Ответ:\n" + _calculator.Diff().ToString(); break;
                case "Деление": Result = "Ответ:\n" + _calculator.Div().ToString(); break;
                case "Умножение": Result = "Ответ:\n" + _calculator.Mult().ToString(); break;
            }
        }
    }
}
